<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       http://www.servicetrade.com
 * @since      1.0.0
 *
 * @package    Servicetrade_Service_Portal
 * @subpackage Servicetrade_Service_Portal/includes
 * 
 * DO NOT REMOVE THE FOLLOWING TWO LINES - MAKES THE UPDATE CHECKER WORK WITH SYMLINKS
 * @wordpress-plugin
 * Plugin Name:       ServiceTrade Service Portal
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Servicetrade_Service_Portal
 * @subpackage Servicetrade_Service_Portal/includes
 * @author     ServiceTrade <support@servicetrade.com>
 */
class Servicetrade_Service_Portal {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Servicetrade_Service_Portal_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $Servicetrade_Service_Portal    The string used to uniquely identify this plugin.
	 */
	protected $Servicetrade_Service_Portal;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		$this->Servicetrade_Service_Portal = 'servicetrade-service-portal';
		$this->version = '2.0.6';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
		$this->disable_admin_bar();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Servicetrade_Service_Portal_Loader. Orchestrates the hooks of the plugin.
	 * - Servicetrade_Service_Portal_i18n. Defines internationalization functionality.
	 * - Servicetrade_Service_Portal_Admin. Defines all hooks for the admin area.
	 * - Servicetrade_Service_Portal_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-portal-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-portal-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'admin/class-servicetrade-portal-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'public/class-servicetrade-portal-public.php';

		$this->loader = new Servicetrade_Service_Portal_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Servicetrade_Service_Portal_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Servicetrade_Service_Portal_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Servicetrade_Service_Portal_Admin( $this->get_Servicetrade_Service_Portal(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		$this->loader->add_action( 'admin_menu', $plugin_admin, 'setup_admin_menu' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'register_settings' );
		// TODO VERSION CHECK
		$this->loader->add_action( 'rest_api_init', $plugin_admin, 'register_rest_controllers' );

		$this->loader->add_action( 'admin_notices', $plugin_admin, 'api_credentials_notice__error' );

		$this->loader->add_filter('update_option_servicetrade_portal_username',$plugin_admin,'on_api_username_password_changed' );
		$this->loader->add_filter('update_option_servicetrade_portal_password',$plugin_admin,'on_api_username_password_changed' );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Servicetrade_Service_Portal_Public( $this->get_Servicetrade_Service_Portal(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

		$this->loader->add_action( 'widgets_init', $plugin_public, 'register_widgets' );
		$this->loader->add_shortcode( 'servicetrade-service-portal', $plugin_public, 'shortcode_servicetrade' );

		// TODO VERSION CHECK
		$this->loader->add_action( 'rest_api_init', $plugin_public, 'register_rest_controllers' );
		
	}

	/**
	 * Disable the admin bar for all non-admins
	 *
	 * @since    2.0.0
	 */
	private function disable_admin_bar() {
		add_action('after_setup_theme', 'remove_admin_bar');
		function remove_admin_bar() {
			if (!current_user_can('administrator') && !is_admin()) {
			  show_admin_bar(false);
			}
		}
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_Servicetrade_Service_Portal() {
		return $this->Servicetrade_Service_Portal;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Servicetrade_Service_Portal_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}
}
