<?php

require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-action.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'includes/class-servicetrade-helper-sideloadmapper.php';
require_once SERVICETRADE_PORTAL_PLUGIN_ROOT_DIR . 'public/rest/class-public-rest-action.php';

/**
 * Returns the deficiencies recorder for an asset.
 *
 * @example
 * http://localhost:8888/wp-json/servicetrade-service-portal/v2/public/locations/1739987/deficiencies
 *
 * @link       http://www.servicetrade.com
 * @since      2.0.0
 * @package    Servicetrade_Service_Portal
 * @subpackage Servicetrade_Service_Portal/public/rest/actions
 * @author     ServiceTrade <support@servicetrade.com>
 */
class Servicetrade_Service_Portal_Public_Actions_Rest_Asset_Deficiencies_Get_Action extends Servicetrade_Service_Portal_Public_Rest_Action {
  private $id;
  private $fixed;

  function __construct($id, $fixed) {
    assert($id !== null, "An id is required.");
    $this->id = $id;
    $this->fixed = $fixed;
  }

  function run() {
    $user = $this->get_authenticated_user();
    if(null === $user) {
			return $this->create_rest_error_not_authorized();
    }
	  $contactId = $user->getContactId();

		$apiResult = $this->api_get($this->build_asset_deficiencies_query());

    if(!isset( $apiResult['deficiencies'] )) {
      return $this->create_rest_error_failed_request();
		}
		
 	  $mapParams = [
		 'asset' => 'assets',
		 'serviceLine' => 'serviceLines',
		 'serviceRequests' => 'serviceRequests',
		 'quote' => 'quotes',
	 ];
	 $deficiencies =  Servicetrade_Helper_Sideloadmapper::map_sideloaded_data( $apiResult, 'deficiencies', $mapParams );

    $mappedResults = array();
	  foreach ($deficiencies as $result) {

		// Filter out invalid
		if($result['status'] === 'invalid') {
			continue;
		}

		// Filter out fixed unless the fixed query parameter is set.
		if(empty($this->fixed)) {
			if($result['status'] === 'fixed') {
				continue;
			}
		}

		 $quotes = [];
		 $lastQuote = $quoteAmount = $encryptedQuoteLinkData = null;
		 if ($result['serviceRequests']) {
			 $quotes = array_map(function($s) { return $s['quote']; }, $result['serviceRequests']);
			 $quotes = array_values(array_filter($quotes, function($q) {
				 return in_array($q['status'], ['submitted', 'accepted']);
			 }));
			 $lastQuote = $quotes ? $quotes[0] : [];
			 $quoteAmount = array_sum(array_map(function($q) { return $q['totalPrice']; }, $quotes));
		 }

		 $statusColor = '';
		 $status = ucwords(str_replace('_', ' ', $result['status']));
		 if ($status == 'Fixed') {
			 $statusColor = 'green';
		 } else if ($status != 'Fixed' && $lastQuote) {
			 $statusColor = 'blue';
			 $status = 'Quoted';
			 $quoteLinkData = ['quoteId' => $lastQuote['id'], 'contactId' => $contactId];
			 $encryptedQuoteLinkData = Servicetrade_Service_Portal_Public_Authenticated_User::encryptUserData($quoteLinkData);
		 } else if (in_array($status, ['New','Verified'])) {
			 $status = 'Open';
		 }

		 if ($result['serviceLine']) {
			 $serviceLines = [[
				 'id' => $result['serviceLine']['id'],
				 'name' => $result['serviceLine']['name'],
			 ]];
		 } else {
			 $serviceLines = [];
		 }

       $description = implode(' - ', array_filter([strtoupper($result['severity']),$result['description']]));

      $mappedResults[] = array(
        'assetId' => $result['asset']['id'],
        'assetUri' => $result['asset']['uri'],
        'assetName' => $result['asset']['name'],
        'description' => $description,
        'reportedOn' => $result['reportedOn'],
        'serviceLines' => $serviceLines,
        'status' => $status,
        'statusColor' => $statusColor,
        'quoteId' => $lastQuote ? $lastQuote['id'] : null,
        'quoteAmount' => $quoteAmount,
        'quoteLinkData' => $encryptedQuoteLinkData,
      );
    }

    $restResponse =  array(
      "deficiencies" => $mappedResults
    );
    return $this->create_rest_success($restResponse);
  } 

	protected function build_asset_deficiencies_query() {
		$query = array( 
			'assetId' => $this->id,
			'_sideload' => 'deficiency.serviceRequests,deficiency.asset,deficiency.serviceLine,serviceRequest.quote'
		);
    return '/deficiency?' . http_build_query($query);
	}

}
